<!DOCTYPE html>
<html lang="en">
<head>
  <title>Paperboy</title>
<link rel="stylesheet" type="text/css" href="{{baseurl}}static/fonts/fonts.css">
<link href="{{baseurl}}static/css/font-awesome.min.css" rel="stylesheet">
<link href='https://fonts.googleapis.com/css?family=Roboto' rel='stylesheet'>
<script type="text/javascript" src="{{baseurl}}static/js/bundle.js"></script>
<script>
    document.user = '{{user}}';
    document.baseurl = '{{ baseurl }}';
    document.apiurl = '{{ apiurl }}';
    document.loginurl = '{{baseurl}}{{loginurl}}';
    document.logouturl = '{{baseurl}}{{logouturl}}';
</script>
</head>
<body>
<div class="footer"><span>Copyright paperboy 2018</span></div>
</body>
</html>