<!DOCTYPE html>
<html lang="en">
<head>
  <title>Paperboy</title>
  <link rel="stylesheet" type="text/css" href="{{baseurl}}static/fonts/fonts.css">
  <link rel="stylesheet" type="text/css" href="{{baseurl}}static/css/login.css">
  <link href="{{baseurl}}static/css/font-awesome.min.css" rel="stylesheet">
  <link href='https://fonts.googleapis.com/css?family=Roboto' rel='stylesheet'>
  <script>
    document.baseurl = '{{baseurl}}';
    document.apiurl = '{{apiurl}}';
</script>
</head>
<body>
  <div class="background"></div>
    <div class="log-form">
        <h2>Are you sure you want to logout?</h2>
        <form action='{{baseurl}}{{logouturl}}' method='post' enctype="multipart/form-data">
            <button type="submit" class="btn">Logout</button>
        </form>
    </div>
</body>
</html>